@@ -2,8 +2,10 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003-???? */
-/* License: GPL */
+/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003-????
+ * License: GPL
+ * Copyright (c) The Exim Maintainers 2016
+ */
 
 /* Code for unpacking MIME containers. Called from acl.c. */
 
@@ -15,9 +17,9 @@
 uschar demime_reason_buffer[1024];
 struct file_extension *file_extensions = NULL;
 
-int demime(uschar **listptr) {
+int demime(const uschar **listptr) {
   int sep = 0;
-  uschar *list = *listptr;
+  const uschar *list = *listptr;
   uschar *option;
   uschar option_buffer[64];
   unsigned long mbox_size;
