@@ -2,8 +2,10 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015 */
-/* License: GPL */
+/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015
+ * License: GPL
+ * Copyright (c) The Exim Maintainers 2016
+ */
 
 /* Code for calling spamassassin's spamd. Called from acl.c. */
 
@@ -139,7 +141,7 @@ long rnd, weights;
 unsigned pri;
 static BOOL srandomed = FALSE;
 
-/* seedup, if we have only 1 server */
+/* speedup, if we have only 1 server */
 if (num_servers == 1)
   return (spamds[0]->is_failed ? -1 : 0);
 
@@ -297,7 +299,7 @@ start = time(NULL);
     sd = (spamd_address_container *)store_get(sizeof(spamd_address_container));
 
     for (sublist = address, args = 0, spamd_param_init(sd);
-	 s = string_nextinlist(&sublist, &sublist_sep, NULL, 0);
+	 (s = string_nextinlist(&sublist, &sublist_sep, NULL, 0));
 	 args++
 	 )
       {
@@ -348,7 +350,7 @@ start = time(NULL);
          || sd->retry <= 0
 	 )
 	break;
-      DEBUG(D_acl) debug_printf("sspamd: erver %s: retry conn\n", sd->hostspec);
+      DEBUG(D_acl) debug_printf("spamd: server %s: retry conn\n", sd->hostspec);
       while (sd->retry > 0) sd->retry = sleep(sd->retry);
       }
     if (spamd_sock >= 0)
@@ -367,13 +369,6 @@ start = time(NULL);
     }
   }
 
-if (spamd_sock == -1)
-  {
-  log_write(0, LOG_MAIN|LOG_PANIC,
-      "programming fault, spamd_sock unexpectedly unset");
-  goto defer;
-  }
-
 (void)fcntl(spamd_sock, F_SETFL, O_NONBLOCK);
 /* now we are connected to spamd on spamd_sock */
 if (sd->is_rspamd)
@@ -397,7 +392,7 @@ if (sd->is_rspamd)
   if ((authid = expand_string(US"$authenticated_id")) != NULL && *authid != '\0')
     req_str = string_sprintf("%sUser: %s\r\n", req_str, authid);
   req_str = string_sprintf("%s\r\n", req_str);
-  wrote = send(spamd_sock, req_str, Ustrlen(req_str), 0); 
+  wrote = send(spamd_sock, req_str, Ustrlen(req_str), 0);
   }
 else
   {				/* spamassassin variant */
@@ -414,7 +409,7 @@ if (wrote == -1)
   {
   (void)close(spamd_sock);
   log_write(0, LOG_MAIN|LOG_PANIC,
-       "%s spamd send failed: %s", loglabel, strerror(errno));
+       "%s spamd %s send failed: %s", loglabel, callout_address, strerror(errno));
   goto defer;
   }
 
@@ -459,13 +454,13 @@ again:
       {
       if (result == -1)
 	log_write(0, LOG_MAIN|LOG_PANIC,
-	  "%s %s on spamd socket", loglabel, strerror(errno));
+	  "%s %s on spamd %s socket", loglabel, callout_address, strerror(errno));
       else
 	{
 	if (time(NULL) - start < sd->timeout)
 	  goto again;
 	log_write(0, LOG_MAIN|LOG_PANIC,
-	  "%s timed out writing spamd socket", loglabel);
+	  "%s timed out writing spamd %s, socket", loglabel, callout_address);
 	}
       (void)close(spamd_sock);
       goto defer;
@@ -475,7 +470,7 @@ again:
     if (wrote == -1)
       {
       log_write(0, LOG_MAIN|LOG_PANIC,
-	  "%s %s on spamd socket", loglabel, strerror(errno));
+	  "%s %s on spamd %s socket", loglabel, callout_address, strerror(errno));
       (void)close(spamd_sock);
       goto defer;
       }
@@ -507,14 +502,15 @@ offset = 0;
 while ((i = ip_recv(spamd_sock,
 		   spamd_buffer + offset,
 		   sizeof(spamd_buffer) - offset - 1,
-		   sd->timeout - time(NULL) + start)) > 0 )
+		   sd->timeout - time(NULL) + start)) > 0)
   offset += i;
+spamd_buffer[offset] = '\0';	/* guard byte */
 
 /* error handling */
 if (i <= 0 && errno != 0)
   {
   log_write(0, LOG_MAIN|LOG_PANIC,
-       "%s error reading from spamd socket: %s", loglabel, strerror(errno));
+       "%s error reading from spamd %s, socket: %s", loglabel, callout_address, strerror(errno));
   (void)close(spamd_sock);
   return DEFER;
   }
@@ -525,13 +521,15 @@ if (i <= 0 && errno != 0)
 if (sd->is_rspamd)
   {				/* rspamd variant of reply */
   int r;
-  if ((r = sscanf(CS spamd_buffer,
+  if (  (r = sscanf(CS spamd_buffer,
 	  "RSPAMD/%7s 0 EX_OK\r\nMetric: default; %7s %lf / %lf / %lf\r\n%n",
 	  spamd_version, spamd_short_result, &spamd_score, &spamd_threshold,
-	  &spamd_reject_score, &spamd_report_offset)) != 5)
+	  &spamd_reject_score, &spamd_report_offset)) != 5
+     || spamd_report_offset >= offset		/* verify within buffer */
+     )
     {
     log_write(0, LOG_MAIN|LOG_PANIC,
-	      "%s cannot parse spamd output: %d", loglabel, r);
+	      "%s cannot parse spamd %s, output: %d", loglabel, callout_address, r);
     return DEFER;
     }
   /* now parse action */
@@ -560,7 +558,7 @@ else
 	   spamd_version,&spamd_score,&spamd_threshold,&spamd_report_offset) != 3)
 	{
 	log_write(0, LOG_MAIN|LOG_PANIC,
-		  "%s cannot parse spamd output", loglabel);
+		  "%s cannot parse spamd %s output", loglabel, callout_address);
 	return DEFER;
 	}
     }
